@@ -1,18 +1,22 @@
-from django.db import models
-from django.contrib.auth.models import User
-from django.conf import settings
-from tracking import utils
 from datetime import datetime, timedelta
-import os
-import unicodedata
+import logging
+import traceback
 
-try:
-    import GeoIP
-except ImportError:
-    GeoIP = None
+from django.contrib.gis.utils import HAS_GEOIP
 
-def u_clean(s):
-    return unicodedata.normalize('NFKD', unicode(s)).encode('ascii', 'ignore')
+if HAS_GEOIP:
+    from django.contrib.gis.utils import GeoIP, GeoIPException
+
+from django.conf import settings
+from django.contrib.auth.models import User
+from django.db import models
+from django.utils.translation import ugettext, ugettext_lazy as _
+from tracking import utils
+
+USE_GEOIP = getattr(settings, 'TRACKING_USE_GEOIP', False)
+CACHE_TYPE = getattr(settings, 'GEOIP_CACHE_TYPE', 4)
+
+log = logging.getLogger('tracking.models')
 
 class VisitorManager(models.Manager):
     def active(self, timeout=None):
@@ -56,29 +60,30 @@ class Visitor(models.Model):
 
             return u'%i:%02i:%02i' % (hours, minutes, seconds)
         else:
-            return u'unknown'
+            return ugettext(u'unknown')
     time_on_site = property(_time_on_site)
 
     def _get_geoip_data(self):
         """
         Attempts to retrieve MaxMind GeoIP data based upon the visitor's IP
         """
-        if getattr(settings, 'TRACKING_USE_GEOIP', False) and GeoIP:
-            geoip_data_file = getattr(settings, 'GEOIP_DATA_FILE', None)
 
-            if geoip_data_file:
-                assert os.access(geoip_data_file, os.R_OK)
-                gip = GeoIP.open(geoip_data_file, GeoIP.GEOIP_MEMORY_CACHE)
-            else:
-                gip = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE)
+        if not HAS_GEOIP or not USE_GEOIP:
+            # go no further when we don't need to
+            log.debug('Bailing out.  HAS_GEOIP: %s; TRACKING_USE_GEOIP: %s' % (HAS_GEOIP, USE_GEOIP))
+            return None
 
+        if not hasattr(self, '_geoip_data'):
+            self._geoip_data = None
             try:
-                return gip.record_by_addr(self.ip_address)
-            except SystemError:
-                # if we get here, chances are that we didn't get a result for
-                # the IP
-                pass
-        return None
+                gip = GeoIP(cache=CACHE_TYPE)
+                self._geoip_data = gip.city(self.ip_address)
+            except GeoIPException:
+                # don't even bother...
+                log.error('Error getting GeoIP data for IP "%s": %s' % (self.ip_address, traceback.format_exc()))
+
+        return self._geoip_data
+
     geoip_data = property(_get_geoip_data)
 
     def _get_geoip_data_json(self):
@@ -87,10 +92,12 @@ class Visitor(models.Model):
         JSON encoding.
         """
         clean = {}
+        if not self.geoip_data: return {}
+
         for key,value in self.geoip_data.items():
-            print 'Cleaning: %s :: %s' % (key, value)
-            clean[key] = u_clean(value)
+            clean[key] = utils.u_clean(value)
         return clean
+
     geoip_data_json = property(_get_geoip_data_json)
 
     class Meta:
@@ -98,23 +105,23 @@ class Visitor(models.Model):
         unique_together = ('session_key', 'ip_address',)
 
 class UntrackedUserAgent(models.Model):
-    keyword = models.CharField(max_length=100, help_text='Part or all of a user-agent string.  For example, "Googlebot" here will be found in "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)" and that visitor will not be tracked.')
+    keyword = models.CharField(_('keyword'), max_length=100, help_text=_('Part or all of a user-agent string.  For example, "Googlebot" here will be found in "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)" and that visitor will not be tracked.'))
 
     def __unicode__(self):
         return self.keyword
 
     class Meta:
         ordering = ('keyword',)
-        verbose_name = 'Untracked User-Agent'
-        verbose_name_plural = 'Untracked User-Agents'
+        verbose_name = _('Untracked User-Agent')
+        verbose_name_plural = _('Untracked User-Agents')
 
 class BannedIP(models.Model):
-    ip_address = models.IPAddressField('IP Address', help_text='The IP address that should be banned')
+    ip_address = models.IPAddressField('IP Address', help_text=_('The IP address that should be banned'))
 
     def __unicode__(self):
         return self.ip_address
 
     class Meta:
         ordering = ('ip_address',)
-        verbose_name = 'Banned IP'
-        verbose_name_plural = 'Banned IPs'
\ No newline at end of file
+        verbose_name = _('Banned IP')
+        verbose_name_plural = _('Banned IPs')
